@@ -29,7 +29,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: L_buffer.c,v 1.13 1997/08/28 00:47:28 pjm Exp $
+ * $Id: L_buffer.c,v 1.15 1997/10/26 05:47:40 pjm Exp $
  */
 
 #include <stdio.h>
@@ -58,6 +58,7 @@ L_BUFFER *L_buffer_create(size_t size) {
     p->size = size;
     p->data = malloc(size);
     p->free = 0;
+    p->wraparound = 1;
     if(p->data != 0) {
       int i;
       for(i = 0; i < size; i++) {
@@ -72,6 +73,20 @@ L_BUFFER *L_buffer_create(size_t size) {
   }
 }
 
+/*
+ * L_buffer_wraparound - turn wraparound on/off for a buffer, note
+ *    that wraparound is turned on by default byu L_buffer_create.
+ */
+
+void L_buffer_wraparound(L_BUFFER *b, int wraparound) {
+  if(b == 0) {
+    nana_error("L_buffer_wraparound: arg1 == 0");
+  } else {
+    b->wraparound = wraparound;
+  }
+}
+
+
 /*
  * L_buffer_putchar - copies a single char into the buffer.
  */
@@ -81,9 +96,15 @@ inline void L_buffer_putchar(L_BUFFER *b, char c) {
     nana_error("L_buffer_putchar: buffer == 0?");
   }
 
-  b->data[b->free++] = c;
-  if(b->free == b->size) {
-    b->free = 0;
+  if(b->wraparound) { /* wrapping around so */
+    b->data[b->free++] = c; /* write the char */
+    if(b->free == b->size) { /* at the end so back to the beginning */
+      b->free = 0;
+    }
+  } else { /* not wrapping around */
+    if(b->free < b->size) {
+      b->data[b->free++] = c;
+    }
   }
 }
 
@@ -105,11 +126,7 @@ inline void L_buffer_puts(L_BUFFER *b, char *s) {
 
   /* just copy into the buffer with wraparound */
   for(i = 0; s[i] != '\0'; i++) {
-    b->data[b->free] = s[i];
-    b->free++;
-    if(b->free == b->size) {
-      b->free = 0;
-    }
+    L_buffer_putchar(b, s[i]);
   }
 }
 
@@ -166,6 +183,13 @@ void L_buffer_dump(L_BUFFER *b, FILE *fp) {
   /* print the banner */
   fprintf(fp, "* L_buffer_dump =\n");
 
+  /* if not wraparound just print it and be done */
+  if(!(b->wraparound)) {
+    fprintf(fp, "%s", b->data);
+    goto end;
+  }
+
+  /* otherwise a bit of searching is in order */
   /* skip '\0' bytes from free upwards to find the oldest message */
   for(i = b->free+1; ; i++) { 
     if(i == b->size) { /* wraparound at end of buffer */ 
@@ -179,7 +203,7 @@ void L_buffer_dump(L_BUFFER *b, FILE *fp) {
       break;
     }
   }
-
+    
   /* now print out in time order */
   do {
     c = b->data[i];
